package cn.com.qmth.examcloud.support.handler.richText;

import cn.com.qmth.examcloud.support.enums.BlockType;
import cn.com.qmth.examcloud.support.handler.richText.bean.BlockBean;
import cn.com.qmth.examcloud.support.handler.richText.bean.SectionBean;
import cn.com.qmth.examcloud.support.handler.richText.bean.SectionCollectionBean;
import com.mysql.cj.util.StringUtils;
import org.apache.commons.lang.StringEscapeUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @Description 复合文本处理器类
 * @Author lideyin
 * @Date 2020/5/15 16:29
 * @Version 1.0
 */
public class ComplexTextHandler implements RichTextHandler {
    private static Log logger = LogFactory.getLog(HtmlTextHandler.class);

    @Override
    public SectionCollectionBean handle(String richText) {
        if (logger.isDebugEnabled()) {
            logger.debug("[HTML-TEXT-HANDLER]--richText: " + richText);
        }

        if (StringUtils.isNullOrEmpty(richText)) {
            return new SectionCollectionBean();
        }

        SectionCollectionBean body = new SectionCollectionBean();
        List<SectionBean> sections = new ArrayList<>();
        // 得到小题题干或者答案行数
        if (org.apache.commons.lang3.StringUtils.isBlank(richText)) {
            return body;
        }

        String[] questionRowStrings = richText.split("</p>");
        for (int i = 0; i < questionRowStrings.length; i++) {
            List<BlockBean> blocks = disposeQuestionBodyOrOption(questionRowStrings[i]);
            if (blocks != null && blocks.size() > 0) {
                SectionBean section = new SectionBean();
                // 将小题题干拆分为Block集合
                section.setBlocks(blocks);
                sections.add(section);
            }
        }
        body.setSections(sections);
        return body;
    }


    private List<BlockBean> disposeQuestionBodyOrOption(String questionRow) {
        List<BlockBean> blocks = new ArrayList<>();
        // 去掉每行里面的<p>,<span>,</span>标签
        questionRow = questionRow.replaceAll("<p>", "").replaceAll("</p>", "").replaceAll("<span>", "")
                .replaceAll("</span>", "").replaceAll("</a>", "");
        String[] questionRowStrings = questionRow.split("<|/>|>");
        for (int i = 0; i < questionRowStrings.length; i++) {
            BlockBean block = new BlockBean(BlockType.text.name());
            String rowStr = questionRowStrings[i];
            // 判断是否有图片
            if (rowStr.startsWith("img")) {
                rowStr = "<" + rowStr + ">";
                Map<String, Object> param = new HashMap<>();
                // 需要继续做截取，取到Parma
                block.setType("image");
                // 获取图片的路径
                List<String> strSrcList = getImg(rowStr, "src");
                if (strSrcList.size() > 0) {
                    String strSrc = strSrcList.get(0).replaceAll("src=\"", "").replaceAll("\"", "");
                    block.setValue(strSrc);
                }
                // 获取图片的高度
                List<String> strHeightList = getImg(rowStr, "height");
                if (strHeightList.size() > 0) {
                    String strHeight = strHeightList.get(0).replaceAll("height=\"", "").replaceAll("\"", "");
                    param.put("height", strHeight);
                }
                // 获取图片的宽度
                List<String> strWidthList = getImg(rowStr, "width");
                if (strHeightList.size() > 0) {
                    String strWidth = strWidthList.get(0).replaceAll("width=\"", "").replaceAll("\"", "");
                    param.put("width", strWidth);
                }
                block.setParam(param);
                blocks.add(block);
            } else if (rowStr.startsWith("a") && rowStr.contains("id") && rowStr.contains("name")) { // 处理音频
                rowStr = "<" + rowStr + ">";
                block.setPlayTime(1);
                block.setType("audio");
                block.setValue(this.getAttrValue(rowStr, "url"));
                blocks.add(block);
            } else {
                block.setType("text");
                if (org.apache.commons.lang3.StringUtils.isNotBlank(rowStr)) {
                    block.setValue(StringEscapeUtils.unescapeHtml(rowStr));
                    blocks.add(block);
                }
            }
        }

        return blocks;
    }

    /**
     * 获取图片里面的路径，长度，宽度
     */
    private List<String> getImg(String s, String str) {
        String regex;
        List<String> list = new ArrayList<>();
        regex = str + "=\"(.*?)\"";
        Pattern pa = Pattern.compile(regex, Pattern.DOTALL);
        Matcher ma = pa.matcher(s);
        while (ma.find()) {
            list.add(ma.group());
        }
        return list;
    }

    private String getAttrValue(String questionStr, String attrName) {
        Pattern aPattern = Pattern.compile("a.*");
        Matcher aMatcher = aPattern.matcher(questionStr);

        if (aMatcher.find()) {
            String idRegex = attrName + "=\".*?\"";
            Pattern idPattern = Pattern.compile(idRegex);
            Matcher idMatcher = idPattern.matcher(aMatcher.group());
            if (idMatcher.find()) {
                return idMatcher.group()
                        .replaceAll(attrName + "=\"", "")
                        .replaceAll("\"", "");
            }
        }

        return "";
    }
}
